#region License 
/**
	This is a NinjaScript implementation of the Ichimoku Clouds indicator.
    Copyright (C) 2011 Troder.wordpress.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/
#endregion

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Chikou Span for Ichimoku Clouds
    /// </summary>
    [Description("Chikou Span for Ichimoku Clouds")]
    public class IchimokuCloudsChikouSpan : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int lag = 26; // Default setting for Lag
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.DarkOrange), PlotStyle.Line, "ChikouSpan"));
            Overlay	= true;
			Displacement = -Lag;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
            ChikouSpan.Set(Close[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ChikouSpan
        {
            get { return Values[0]; }
        }

        [Description("Lag period for Chikou Plotting")]
        [GridCategory("Parameters")]
        public int Lag
        {
            get { return lag; }
            set { lag = Math.Max(1, value); Displacement = -value; }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private IchimokuCloudsChikouSpan[] cacheIchimokuCloudsChikouSpan = null;

        private static IchimokuCloudsChikouSpan checkIchimokuCloudsChikouSpan = new IchimokuCloudsChikouSpan();

        /// <summary>
        /// Chikou Span for Ichimoku Clouds
        /// </summary>
        /// <returns></returns>
        public IchimokuCloudsChikouSpan IchimokuCloudsChikouSpan(int lag)
        {
            return IchimokuCloudsChikouSpan(Input, lag);
        }

        /// <summary>
        /// Chikou Span for Ichimoku Clouds
        /// </summary>
        /// <returns></returns>
        public IchimokuCloudsChikouSpan IchimokuCloudsChikouSpan(Data.IDataSeries input, int lag)
        {
            if (cacheIchimokuCloudsChikouSpan != null)
                for (int idx = 0; idx < cacheIchimokuCloudsChikouSpan.Length; idx++)
                    if (cacheIchimokuCloudsChikouSpan[idx].Lag == lag && cacheIchimokuCloudsChikouSpan[idx].EqualsInput(input))
                        return cacheIchimokuCloudsChikouSpan[idx];

            lock (checkIchimokuCloudsChikouSpan)
            {
                checkIchimokuCloudsChikouSpan.Lag = lag;
                lag = checkIchimokuCloudsChikouSpan.Lag;

                if (cacheIchimokuCloudsChikouSpan != null)
                    for (int idx = 0; idx < cacheIchimokuCloudsChikouSpan.Length; idx++)
                        if (cacheIchimokuCloudsChikouSpan[idx].Lag == lag && cacheIchimokuCloudsChikouSpan[idx].EqualsInput(input))
                            return cacheIchimokuCloudsChikouSpan[idx];

                IchimokuCloudsChikouSpan indicator = new IchimokuCloudsChikouSpan();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Lag = lag;
                Indicators.Add(indicator);
                indicator.SetUp();

                IchimokuCloudsChikouSpan[] tmp = new IchimokuCloudsChikouSpan[cacheIchimokuCloudsChikouSpan == null ? 1 : cacheIchimokuCloudsChikouSpan.Length + 1];
                if (cacheIchimokuCloudsChikouSpan != null)
                    cacheIchimokuCloudsChikouSpan.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheIchimokuCloudsChikouSpan = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Chikou Span for Ichimoku Clouds
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IchimokuCloudsChikouSpan IchimokuCloudsChikouSpan(int lag)
        {
            return _indicator.IchimokuCloudsChikouSpan(Input, lag);
        }

        /// <summary>
        /// Chikou Span for Ichimoku Clouds
        /// </summary>
        /// <returns></returns>
        public Indicator.IchimokuCloudsChikouSpan IchimokuCloudsChikouSpan(Data.IDataSeries input, int lag)
        {
            return _indicator.IchimokuCloudsChikouSpan(input, lag);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Chikou Span for Ichimoku Clouds
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IchimokuCloudsChikouSpan IchimokuCloudsChikouSpan(int lag)
        {
            return _indicator.IchimokuCloudsChikouSpan(Input, lag);
        }

        /// <summary>
        /// Chikou Span for Ichimoku Clouds
        /// </summary>
        /// <returns></returns>
        public Indicator.IchimokuCloudsChikouSpan IchimokuCloudsChikouSpan(Data.IDataSeries input, int lag)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.IchimokuCloudsChikouSpan(input, lag);
        }
    }
}
#endregion
