#region License 
/**
	This is a NinjaScript implementation of the Ichimoku Clouds indicator.
    Copyright (C) 2011 Troder.wordpress.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/
#endregion
	
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
	/// 
	/// Ref http://troder.wordpress.com/2011/12/23/ichimoku-clouds-in-ninjatrader/ for report bugs/suggestions
	/// Last Updated: 2011-12-23
    /// </summary>
    [Description("Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud")]
    public class IchimokuCloudsTenkanKijunSen : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int tenkanPeriod = 9;
		private int kijunPeriod = 26;
		
		// User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "Tenkan-sen"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "Kijun-sen"));
			
            Overlay	= true;
			//No plot until the max of all bars are available
			BarsRequired = System.Math.Max(tenkanPeriod,kijunPeriod);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBars[0] <= BarsRequired)
				return;
			
			//Calculate plot values
			double vTenkanSen = (MAX(High,TenkanPeriod)[0] + MIN(Low,TenkanPeriod)[0])/2;
			double vKijunSen = (MAX(High,KijunPeriod)[0] + MIN(Low,KijunPeriod)[0])/2;
			
			//Update plot values
			TenkanSen.Set(vTenkanSen);
			KijunSen.Set(vKijunSen);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TenkanSen
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries KijunSen
        {
            get { return Values[1]; }
        }

        [Description("Tenkan/Conversion Line Period")]
        [GridCategory("Parameters")]
        public int TenkanPeriod
        {
            get { return tenkanPeriod; }
            set { tenkanPeriod = Math.Max(1, value); }
        }
		
		[Description("Kijun/Base Line Period")]
        [GridCategory("Parameters")]
        public int KijunPeriod
        {
            get { return kijunPeriod; }
            set { kijunPeriod = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private IchimokuCloudsTenkanKijunSen[] cacheIchimokuCloudsTenkanKijunSen = null;

        private static IchimokuCloudsTenkanKijunSen checkIchimokuCloudsTenkanKijunSen = new IchimokuCloudsTenkanKijunSen();

        /// <summary>
        /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
        /// </summary>
        /// <returns></returns>
        public IchimokuCloudsTenkanKijunSen IchimokuCloudsTenkanKijunSen(int kijunPeriod, int tenkanPeriod)
        {
            return IchimokuCloudsTenkanKijunSen(Input, kijunPeriod, tenkanPeriod);
        }

        /// <summary>
        /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
        /// </summary>
        /// <returns></returns>
        public IchimokuCloudsTenkanKijunSen IchimokuCloudsTenkanKijunSen(Data.IDataSeries input, int kijunPeriod, int tenkanPeriod)
        {
            if (cacheIchimokuCloudsTenkanKijunSen != null)
                for (int idx = 0; idx < cacheIchimokuCloudsTenkanKijunSen.Length; idx++)
                    if (cacheIchimokuCloudsTenkanKijunSen[idx].KijunPeriod == kijunPeriod && cacheIchimokuCloudsTenkanKijunSen[idx].TenkanPeriod == tenkanPeriod && cacheIchimokuCloudsTenkanKijunSen[idx].EqualsInput(input))
                        return cacheIchimokuCloudsTenkanKijunSen[idx];

            lock (checkIchimokuCloudsTenkanKijunSen)
            {
                checkIchimokuCloudsTenkanKijunSen.KijunPeriod = kijunPeriod;
                kijunPeriod = checkIchimokuCloudsTenkanKijunSen.KijunPeriod;
                checkIchimokuCloudsTenkanKijunSen.TenkanPeriod = tenkanPeriod;
                tenkanPeriod = checkIchimokuCloudsTenkanKijunSen.TenkanPeriod;

                if (cacheIchimokuCloudsTenkanKijunSen != null)
                    for (int idx = 0; idx < cacheIchimokuCloudsTenkanKijunSen.Length; idx++)
                        if (cacheIchimokuCloudsTenkanKijunSen[idx].KijunPeriod == kijunPeriod && cacheIchimokuCloudsTenkanKijunSen[idx].TenkanPeriod == tenkanPeriod && cacheIchimokuCloudsTenkanKijunSen[idx].EqualsInput(input))
                            return cacheIchimokuCloudsTenkanKijunSen[idx];

                IchimokuCloudsTenkanKijunSen indicator = new IchimokuCloudsTenkanKijunSen();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.KijunPeriod = kijunPeriod;
                indicator.TenkanPeriod = tenkanPeriod;
                Indicators.Add(indicator);
                indicator.SetUp();

                IchimokuCloudsTenkanKijunSen[] tmp = new IchimokuCloudsTenkanKijunSen[cacheIchimokuCloudsTenkanKijunSen == null ? 1 : cacheIchimokuCloudsTenkanKijunSen.Length + 1];
                if (cacheIchimokuCloudsTenkanKijunSen != null)
                    cacheIchimokuCloudsTenkanKijunSen.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheIchimokuCloudsTenkanKijunSen = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IchimokuCloudsTenkanKijunSen IchimokuCloudsTenkanKijunSen(int kijunPeriod, int tenkanPeriod)
        {
            return _indicator.IchimokuCloudsTenkanKijunSen(Input, kijunPeriod, tenkanPeriod);
        }

        /// <summary>
        /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
        /// </summary>
        /// <returns></returns>
        public Indicator.IchimokuCloudsTenkanKijunSen IchimokuCloudsTenkanKijunSen(Data.IDataSeries input, int kijunPeriod, int tenkanPeriod)
        {
            return _indicator.IchimokuCloudsTenkanKijunSen(input, kijunPeriod, tenkanPeriod);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IchimokuCloudsTenkanKijunSen IchimokuCloudsTenkanKijunSen(int kijunPeriod, int tenkanPeriod)
        {
            return _indicator.IchimokuCloudsTenkanKijunSen(Input, kijunPeriod, tenkanPeriod);
        }

        /// <summary>
        /// Implementation of Ichimoku Clouds as described at http://stockcharts.com/school/doku.php?id=chart_school:technical_indicators:ichimoku_cloud
        /// </summary>
        /// <returns></returns>
        public Indicator.IchimokuCloudsTenkanKijunSen IchimokuCloudsTenkanKijunSen(Data.IDataSeries input, int kijunPeriod, int tenkanPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.IchimokuCloudsTenkanKijunSen(input, kijunPeriod, tenkanPeriod);
        }
    }
}
#endregion
